import sys

from pathlib import Path

if sys.version_info >= (3, 8):
    from importlib import metadata
else:
    import importlib_metadata as metadata


__version__ = metadata.version(__name__)

GST_PLUGIN_PATH = str(Path(__file__).parent / "plugins")



import typing as t
import functools

import click
import click_completion
import click_completion.core

from . import __version__, GST_PLUGIN_PATH


# monkey patch click to support auto completion
click_completion.init()

# monkey patch click option to show defaults by default
click.option = functools.partial(click.option, show_default=True)


def show_completion(
    ctx: click.Context,
    param: click.Parameter,
    value: t.Any,
) -> None:
    """
    Show auto completion and exit.
    """
    if not value or ctx.resilient_parsing:
        return
    click.echo(click_completion.core.get_code())
    ctx.exit()


def install_completion(
    ctx: click.Context,
    param: click.Parameter,
    value: t.Any,
) -> None:
    """
    Install auto completion and exit.
    """
    if not value or ctx.resilient_parsing:
        return
    shell, path = click_completion.core.install()
    click.echo(f"{shell} completion installed at {path}")
    ctx.exit()


def print_version(
    ctx: click.Context,
    param: click.Parameter,
    value: t.Any,
) -> None:
    """
    Show version and quit.
    """
    if not value or ctx.resilient_parsing:
        return
    click.echo(__version__)
    ctx.exit()


def print_gstreamer_plugin_path(
    ctx: click.Context,
    param: click.Parameter,
    value: t.Any,
) -> None:
    """
    Show Gstremaer plugin path and exit.
    """
    if not value or ctx.resilient_parsing:
        return
    click.echo(GST_PLUGIN_PATH)
    ctx.exit()



import typing as t

from .click import (
    click,
    print_version,
    print_gstreamer_plugin_path,
    install_completion,
    show_completion,
)
from .mediapipe import (
    SelfieSegmentationModel,
)
from .gst import (
  add_filters,
  print_device_caps,
)


@click.command()
@click.option(
    "--input-dev",
    help="Input device (e.g. real webcam at /dev/video0).",
    type=str,
    required=True,
)
@click.option(
    "--input-width",
    help="Input width.",
    type=int,
    default=None,
)
@click.option(
    "--input-height",
    help="Input height.",
    type=int,
    default=None,
)
@click.option(
    "--input-framerate",
    help="Input framerate specified in fractional format (e.g. '30/1').",
    type=str,
    default=None,
)
@click.option(
    "--input-media-type",
    help="""
    Input media type (e.g. 'image/jpeg' or 'x-raw-video').
    Use --list-dev-caps to see all available formats.
    """,
    type=str,
    default=None,
)
@click.option(
    "--output-dev",
    help="Output device (e.g. virtual webcam at /dev/video3).",
    type=str,
    required=True,
)
@click.option(
    "--background-blur",
    help="Background blur intensity.",
    type=click.IntRange(0, 200),
    default=None,
)
@click.option(
    "--selfie-segmentation-model",
    help="Mediapipe model used for selfie segmentation",
    type=click.Choice([x.name for x in SelfieSegmentationModel]),
    default=SelfieSegmentationModel.general.name,
)
@click.option(
    "--selfie-segmentation-threshold",
    help="Selfie segmentation threshold.",
    type=click.FloatRange(min=0, max=1),
    default=0.5,
)
@click.option(
    "--list-dev-caps",
    help="List device capabilities (width, height, framerate, etc) and exit.",
    type=str,
    is_eager=True,
    callback=print_device_caps,
    show_default=False,
)
@click.option(
    "--gst-plugin-path",
    help="Show Gstreamer plugin path and exit.",
    is_flag=True,
    is_eager=True,
    callback=print_gstreamer_plugin_path,
    expose_value=False,
    show_default=False,
)
@click.option(
    "--install-completion",
    help="Install auto completion for the current shell and exit.",
    is_flag=True,
    is_eager=True,
    callback=install_completion,
    expose_value=False,
    show_default=False,
)
@click.option(
    "--show-completion",
    help="Show auto completion for the current shell and exit.",
    is_flag=True,
    is_eager=True,
    callback=show_completion,
    expose_value=False,
    show_default=False,
)
@click.option(
    "--version",
    help="Show version and exit.",
    is_flag=True,
    is_eager=True,
    callback=print_version,
    expose_value=False,
    show_default=False,
)
def cli(
    input_dev: str,
    input_width: t.Optional[int],
    input_height: t.Optional[int],
    input_framerate: t.Optional[str],
    input_media_type: t.Optional[str],
    output_dev: str,
    background_blur: t.Optional[int],
    selfie_segmentation_model: str,
    selfie_segmentation_threshold: int,
    **kwargs
) -> None:
    """
    Add video filters to your webcam.
    """
    selfie_seg_model = SelfieSegmentationModel[selfie_segmentation_model]
    add_filters(
        input_dev,
        output_dev,
        input_width,
        input_height,
        input_framerate,
        input_media_type,
        background_blur,
        selfie_seg_model,
        selfie_segmentation_threshold,
    )



import gi

gi.require_version("Gst", "1.0")
gi.require_version("GstBase", "1.0")

import numpy

from gi.repository import Gst, GstBase, GLib, GObject

from mediapipe.python.solutions.selfie_segmentation import SelfieSegmentation


GObject.threads_init()
Gst.init(None)

SRC_CAPS = Gst.Caps(Gst.Structure(
    "video/x-raw",
    format="RGB",
    width=Gst.IntRange(range(1, GLib.MAXINT)),
    height=Gst.IntRange(range(1, GLib.MAXINT)),
    framerate=Gst.FractionRange(
        Gst.Fraction(1, 1),
        Gst.Fraction(GLib.MAXINT, 1)
    )
))

SINK_CAPS = Gst.Caps(Gst.Structure(
    "video/x-raw",
    format="RGB",
    width=Gst.IntRange(range(1, GLib.MAXINT)),
    height=Gst.IntRange(range(1, GLib.MAXINT)),
    framerate=Gst.FractionRange(
        Gst.Fraction(1, 1),
        Gst.Fraction(GLib.MAXINT, 1)
    )
))

SRC_PAD_TEMPLATE = Gst.PadTemplate.new(
    "src",
    Gst.PadDirection.SRC,
    Gst.PadPresence.ALWAYS,
    SRC_CAPS
)

SINK_PAD_TEMPLATE = Gst.PadTemplate.new(
    "sink",
    Gst.PadDirection.SINK,
    Gst.PadPresence.ALWAYS,
    SINK_CAPS
)

DEFAULT_MODEL = 0
DEFAULT_THRESHOLD = 0.5


class SelfieSegmenter(GstBase.BaseTransform):

    __gstmetadata__ = (
        "Selfie Segmenter",
        "Filter",
        "Perform selfie segmentation using mediapipe model",
        "Jashandeep Sohi <jashandeep.s.sohi@gmail.com>"
    )

    __gsttemplates__ = (SRC_PAD_TEMPLATE, SINK_PAD_TEMPLATE)

    __gproperties__ = {
        "model": (
            int,
            "Segmentation model",
            "Segmentation model (0=general, 1=landscape)",
            0,
            1,
            DEFAULT_MODEL,
            GObject.ParamFlags.READWRITE
        ),
        "threshold": (
            float,
            "Segmentation threshold",
            "Segmentation  threshold",
            0.0,
            1.0,
            DEFAULT_THRESHOLD,
            GObject.ParamFlags.READWRITE
        )
    }

    def __init__(self):
        super().__init__()
        self.model = DEFAULT_MODEL
        self.threshold = DEFAULT_THRESHOLD
        self.set_qos_enabled(True)

    def do_get_property(self, prop: GObject.GParamSpec):
        if prop.name == "model":
            return self.model
        elif prop.name == "threshold":
            return self.threshold
        else:
            raise AttributeError(f"unkown property {prop.name}")

    def do_set_property(self, prop: GObject.GParamSpec, value):
        if prop.name == "model":
            self.model  = value
        elif prop.name == "threshold":
            self.threshold  = value
        else:
            raise AttributeError(f"unkown property {prop.name}")

    def do_start(self):
        self.mp_seg = SelfieSegmentation(model_selection=self.model)

        return True

    def do_transform_caps(
        self,
        direction: Gst.PadDirection,
        caps: Gst.Caps,
        filter_: Gst.Caps
    ) -> Gst.Caps:
        """
         Given a pad in this direction and the given caps, what caps are
         allowed on the other pad in this element.
        """
        if direction == Gst.PadDirection.SRC:
            outcaps = SRC_CAPS
        else:
            outcaps = SINK_CAPS

        if filter_:
            outcaps = outcaps.intersect(filter_)

        return outcaps

    def do_fixate_caps(self, direction, caps, othercaps):
        if direction == Gst.PadDirection.SRC:
            return othercaps.fixate()
        else:
            return caps.fixate()

    def do_set_caps(self, incaps, outcaps):
        s = incaps.get_structure(0)
        self.width = s.get_int("width").value
        self.height = s.get_int("height").value

        return True

    def do_transform_ip(self, inbuf):
        try:
            inbuf_info = inbuf.map(Gst.MapFlags.READ | Gst.MapFlags.WRITE)
            with inbuf_info:
                in_nd = numpy.ndarray(
                    shape=(self.height, self.width, 3),
                    dtype=numpy.uint8,
                    buffer=inbuf_info.data
                )

                in_nd.flags.writeable = False
                result = self.mp_seg.process(in_nd)
                in_nd.flags.writeable = True
                self.mp_seg.reset()

                mask = -(result.segmentation_mask >= self.threshold).view("uint8")

                numpy.stack((mask, mask, mask), -1, in_nd)

                return Gst.FlowReturn.OK

        except Gst.MapError as e:
            Gst.error("mapping error %s" % e)
            return Gst.FlowReturn.ERROR
        except Exception as e:
            Gst.error("%s" % e)
            return Gst.FlowReturn.ERROR

GObject.type_register(SelfieSegmenter)

__gstelementfactory__ = (
    "selfie_seg",
    Gst.Rank.NONE,
    SelfieSegmenter
)



import gi
import time

gi.require_version("Gst", "1.0")
gi.require_version("GstBase", "1.0")

import numpy

from gi.repository import Gst, GstBase, GLib, GObject


GObject.threads_init()
Gst.init(None)


SRC_CAPS = Gst.Caps(Gst.Structure(
    "video/x-raw",
    format="RGB",
    width=Gst.IntRange(range(1, GLib.MAXINT)),
    height=Gst.IntRange(range(1, GLib.MAXINT)),
    framerate=Gst.FractionRange(
        Gst.Fraction(1, 1),
        Gst.Fraction(GLib.MAXINT, 1)
    )
))

SINK_CAPS = Gst.Caps(Gst.Structure(
    "video/x-raw",
    format="RGB",
    width=Gst.IntRange(range(1, GLib.MAXINT)),
    height=Gst.IntRange(range(1, GLib.MAXINT)),
    framerate=Gst.FractionRange(
        Gst.Fraction(1, 1),
        Gst.Fraction(GLib.MAXINT, 1)
    )
))

SRC_PAD_TEMPLATE = Gst.PadTemplate.new_with_gtype(
    "src",
    Gst.PadDirection.SRC,
    Gst.PadPresence.ALWAYS,
    SRC_CAPS,
    GstBase.AggregatorPad.__gtype__
)


SINK_PAD_TEMPLATE = Gst.PadTemplate.new_with_gtype(
    "sink",
    Gst.PadDirection.SINK,
    Gst.PadPresence.ALWAYS,
    SINK_CAPS,
    GstBase.AggregatorPad.__gtype__
)


class Where(GstBase.Aggregator):

    __gstmetadata__ = (
        "OpenCV Boxfilter",
        "Filter",
        "Apply boxfilter",
        "Jashandeep Sohi <jashandeep.s.sohi@gmail.com>"
    )

    __gsttemplates__ = (
        SRC_PAD_TEMPLATE,
        SINK_PAD_TEMPLATE,
    )

    def __init__(self):
        super().__init__()

        self._condition =Gst.Pad.new_from_template(SINK_PAD_TEMPLATE, "condition")
        self._x = Gst.Pad.new_from_template(SINK_PAD_TEMPLATE, "x")
        self._y = Gst.Pad.new_from_template(SINK_PAD_TEMPLATE, "y")

        self.add_pad(self._condition)
        self.add_pad(self._x)
        self.add_pad(self._y)

        self._allocator, self._allocator_params = self.get_allocator()

    def do_negotiated_src_caps(self, caps):
        s = caps.get_structure(0)
        self._width = s.get_int("width").value
        self._height = s.get_int("height").value

        self._resbuf = Gst.Buffer.new_allocate(
            self._allocator,
            self._width * self._height * 3,
            self._allocator_params,
        )

        return True

    def do_fixate_src_caps(self, caps):
        return self._condition.get_current_caps()

    def do_aggregate(self, timeout):
        try:
            cbuf = self._condition.pop_buffer()
            xbuf = self._x.pop_buffer()
            ybuf = self._y.pop_buffer()
            resbuf = self._resbuf

            resbuf.pts = cbuf.pts
            resbuf.dts = cbuf.dts

            self.selected_samples(cbuf.pts, cbuf.dts, cbuf.duration, None)

            cbuf_info = cbuf.map(Gst.MapFlags.READ)
            xbuf_info = xbuf.map(Gst.MapFlags.READ)
            ybuf_info = ybuf.map(Gst.MapFlags.READ)
            resbuf_info = resbuf.map(Gst.MapFlags.WRITE)

            with cbuf_info, xbuf_info, ybuf_info, resbuf_info:
                condition = numpy.ndarray(
                    shape=(self._height, self._width, 3),
                    dtype=numpy.dtype("bool"),
                    buffer=cbuf_info.data
                )
                x = numpy.ndarray(
                    shape=(self._height, self._width, 3),
                    dtype=numpy.uint8,
                    buffer=xbuf_info.data
                )
                y = numpy.ndarray(
                    shape=(self._height, self._width, 3),
                    dtype=numpy.uint8,
                    buffer=ybuf_info.data
                )
                res = numpy.ndarray(
                    shape=(self._height, self._width, 3),
                    dtype=numpy.uint8,
                    buffer=resbuf_info.data
                )

                res[:] = numpy.where(condition, x, y)

            self.finish_buffer(resbuf)

            return Gst.FlowReturn.OK

        except Gst.MapError as e:
            Gst.error("mapping error %s" % e)
            return Gst.FlowReturn.ERROR
        except Exception as e:
            Gst.error("%s" % e)
            return Gst.FlowReturn.ERROR

GObject.type_register(Where)

__gstelementfactory__ = (
    "numpy_where",
    Gst.Rank.NONE,
    Where
)



import typing as t
import logging
import os
import sys

import gi

gi.require_version("Gst", "1.0")
gi.require_version("GstBase", "1.0")

from fractions import Fraction
from pathlib import Path

from gi.repository import Gst, GObject, GstBase
from rich.console import Console
from rich.table import Table

from .mediapipe import (
    SelfieSegmentationModel,
)
from .click import (
    click,
)

from . import GST_PLUGIN_PATH


logger = logging.getLogger(__name__)
logger.setLevel(logging.INFO)


# hack to let Gst.ElementFactory.make("...") to find custom plugins
os.environ["GST_PLUGIN_PATH"] = ":".join(
    [GST_PLUGIN_PATH] + os.environ.get("GST_PLUGIN_PATH", "").split(":")
)

# hack to ensure interperter path is on PATH so that custom plugins will use the correct interpeter
os.environ["PATH"] = ":".join(
    [str(Path(sys.executable).parent)] + os.environ.get("PATH", "").split(":")
)

GObject.threads_init()
Gst.init(None)


def add_filters(
    input_dev: str,
    output_dev: str,
    input_width: t.Optional[int],
    input_height: t.Optional[int],
    input_framerate: t.Optional[str],
    input_media_type: t.Optional[str],
    background_blur: t.Optional[int],
    selfie_segmentation_model: SelfieSegmentationModel,
    selfie_segmentation_threshold: int,
) -> None:
    """
    Run filters pipeline.
    """
    src = Gst.ElementFactory.make("v4l2src")
    src.set_property("device", input_dev)
    src.set_state(Gst.State.READY)
    caps = src.get_static_pad("src").query_caps(None)

    src.set_state(Gst.State.NULL)

    structs = []
    for c in caps:
        keep = True

        if input_media_type is not None and input_media_type != c.get_name():
            keep = False

        if input_width is not None and input_width != c.get_value("width"):
            keep = False

        if input_height is not None and input_height != c.get_value("height"):
            keep = False

        if input_framerate is not None:
            if input_framerate != str(c.get_value("framerate")):
                keep = False

        if keep:
            structs.append(c)

    structs = sorted(
        structs,
        key=lambda x: (
            Fraction(str(x.get_value("framerate"))),
            x.get_value("width"),
            x.get_value("height"),
        ),
        reverse=True
    )
    new_caps = Gst.Caps.new_empty()
    for s in structs:
        new_caps.append_structure(s)

    pipeline = Gst.Pipeline.new()

    inputfilter = Gst.ElementFactory.make("capsfilter")
    decodebin = Gst.ElementFactory.make("decodebin")
    rgbconvert = Gst.ElementFactory.make("videoconvert")
    rgbfilter = Gst.ElementFactory.make("capsfilter")
    tee = Gst.ElementFactory.make("tee")
    sinkconvert = Gst.ElementFactory.make("videoconvert")
    sinkfilter = Gst.ElementFactory.make("capsfilter")
    sink = Gst.ElementFactory.make("v4l2sink")

    inputfilter.set_property("caps", new_caps)
    rgbfilter.set_property(
        "caps",
        Gst.Caps.from_string("video/x-raw, format=RGB")
    )
    sinkfilter.set_property(
        "caps",
        Gst.Caps.from_string(f"video/x-raw, format=YUY2")
    )
    sink.set_property("device", output_dev)
    sink.set_property("throttle-time", 10)
    sink.set_property("qos", True)

    pipeline.add(
        src,
        inputfilter,
        decodebin,
        rgbconvert,
        rgbfilter,
        tee,
        sinkconvert,
        sinkfilter,
        sink,
    )

    src.link(inputfilter)
    inputfilter.link(decodebin)
    decodebin.connect(
        "pad-added",
        lambda dbin, pad: pad.link(rgbconvert.get_static_pad("sink"))
    )
    rgbconvert.link(rgbfilter)
    rgbfilter.link(tee)

    out = tee

    need_selfie = background_blur or False

    if need_selfie:
        selfie_queue = Gst.ElementFactory.make("queue", "selfie_queue")
        selfie = Gst.ElementFactory.make("selfie_seg")
        selfie.set_property("model", selfie_segmentation_model)
        selfie.set_property("threshold", selfie_segmentation_threshold)
        pipeline.add(selfie_queue, selfie)
        tee.link(selfie_queue)
        selfie_queue.link(selfie)

    if background_blur:
        blured_queue = Gst.ElementFactory.make("queue", "blured_queue")
        blured = Gst.ElementFactory.make("cv2_boxfilter")
        blured.set_property("ksize", background_blur)
        pipeline.add(blured_queue, blured)
        tee.link(blured_queue)
        blured_queue.link(blured)

        where = Gst.ElementFactory.make("numpy_where")
        pipeline.add(where)
        selfie.get_static_pad("src").link(where.get_static_pad("condition"))
        tee.request_pad(tee.get_pad_template("src_%u"), None, None).link(
            where.get_static_pad("x")
        )
        blured.get_static_pad("src").link(where.get_static_pad("y"))

        out = where

    out.link(sinkconvert)
    sinkconvert.link(sinkfilter)
    sinkfilter.link(sink)


    loop = GObject.MainLoop()
    bus = pipeline.get_bus()
    bus.add_signal_watch()
    bus.connect("message", on_bus_message, loop)

    pipeline.set_state(Gst.State.PLAYING)
    try:
        loop.run()
    except KeyboardInterrupt:
        loop.quit()

    pipeline.set_state(Gst.State.NULL)


def on_bus_message(
    bus: Gst.Bus,
    message: Gst.Message,
    loop: GObject.MainLoop
) -> bool:
    mtype = message.type

    if mtype == Gst.MessageType.EOS:
        loop.quit()
    elif mtype == Gst.MessageType.ERROR:
        logger.error("%s: %s", *message.parse_error())
        loop.quit()
    elif mtype == Gst.MessageType.WARNING:
        logger.warning("%s: %s", *message.parse_warning())
    elif mtype == Gst.MessageType.INFO:
        logger.info("%s: %s", *message.parse_info())

    return True


def print_device_caps(
    ctx: click.Context,
    param: click.Parameter,
    value: str,
) -> None:
    """
    Print device capabilities and exit.
    """
    if not value or ctx.resilient_parsing:
        return

    src = Gst.ElementFactory.make("v4l2src")
    src.set_property("device", value)

    src.set_state(Gst.State.READY)
    res = src.get_state(1000)
    pad = src.get_static_pad("src")

    if res.state != Gst.State.READY or pad is None:
        click.echo(f"unable to determine capabilities for device {value!r}")
        ctx.exit(1)

    caps = pad.query_caps(None)
    src.set_state(Gst.State.NULL)

    if caps.is_any():
        click.echo(f"unable to determine capabilities for device {value!r}")
        ctx.exit(1)

    table = Table(title=f"{value!r} Capabilites")

    table.add_column("Media Type")
    table.add_column("Width")
    table.add_column("Height")
    table.add_column("Framerate")

    for c in caps:
        table.add_row(
            c.get_name(),
            str(c.get_value("width")),
            str(c.get_value("height")),
            str(c.get_value("framerate")),
        )

    console = Console()
    console.print(table)

    ctx.exit(0)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['webcam_filters', 'webcam_filters.plugins.python']

package_data = \
{'': ['*']}

install_requires = \
['click-completion>=0.5.2,<0.6.0',
 'click>=8.0.1,<9.0.0',
 'mediapipe>=0.8.6,<0.9.0',
 'numpy>=1.21.2,<2.0.0',
 'opencv-contrib-python>=4.5.3,<5.0.0',
 'rich>=10.7.0,<11.0.0']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>=1.0']}

entry_points = \
{'console_scripts': ['webcam-filters = webcam_filters.main:cli']}

setup_kwargs = {
    'name': 'webcam-filters',
    'version': '0.2.0',
    'description': 'Add filters (background blur, etc) to your webcam on Linux',
    'long_description': '|pypi-badge|\n\nwebcam-filters\n==============\n\nAdd filters (background blur, etc) to your webcam on Linux.\n\nVideo conferencing applications tend to either lack video effects altogether or\nsupport only a limited set of capabilities on Linux (e.g. Zoom, Google Meets).\n\nGoal here is to provide a virtual webcam via ``v4l2loopback`` with a common\nset of filters that can be used everywhere.\n\nUsage\n-----\nPassthrough (no-op)::\n\n  $ webcam-filters --input-dev /dev/video0 --output-dev /dev/video3\n\nBlur background::\n\n  $ webcam-filters --input-dev /dev/video0 --output-dev /dev/video3 --background-blur 150\n\nDependencies\n------------\nOther than the Python dependencies that can be automatically installed by Pip,\nthere are a few system dependencies that require manual attention.\n\nv4l2loopback\n************\n`v4l2loopback` kernel module is required to emulate a virtual webcam. See your\ndistro\'s docs or v4l2loopback_ on how to install and set it up\n(e.g. https://archlinux.org/packages/community/any/v4l2loopback-dkms/).\n\nYou\'ll probably want to create at least one loopback device (that\'s persistent\non boot)::\n\n  $ sudo tee /etc/modprobe.d/v4l2loopback.conf << "EOF"\n  # /dev/video3\n  options v4l2loopback video_nr=3\n  options v4l2loopback card_label="Virtual Webcam"\n  options v4l2loopback exclusive_caps=1\n  EOF\n  $ sudo modprobe v4l2loopback\n  $ v4l2-ctl --device /dev/video3 --info\n\nGstreamer\n*********\n\n- gstreamer-1.0 (e.g. https://archlinux.org/packages/extra/x86_64/gstreamer/)\n- gst-plugins-base (e.g. https://archlinux.org/packages/extra/x86_64/gst-plugins-base/)\n- gst-python (e.g. https://archlinux.org/packages/extra/x86_64/gst-python/)\n\n\nInstallation\n------------\nYou can either use `pipx` or `pip`. Pipx_ is recommend to keep dependencies\nisolated.\n\nLatest stable::\n\n  $ pipx install --system-site-packages webcam-filters\n  $ pip install --user webcam-filters\n\nLatest pre-release::\n\n  $ pipx install --system-site-packages --pip-args=\'--pre\' webcam-filters\n  $ pip install --user --pre webcam-filters\n\nGit::\n\n  $ url="git+https://github.com/jashandeep-sohi/webcam-filters.git"\n  $ pipx install --system-site-packages "$url"\n  $ pip install --user "$url"\n\n\n.. _Pipx: https://github.com/pypa/pipx\n\n.. _v4l2loopback_: https://github.com/umlaeute/v4l2loopback\n\n.. |pypi-badge| image:: https://img.shields.io/pypi/v/webcam-filters\n    :alt: PyPI\n    :target: https://pypi.org/project/webcam-filters/\n',
    'author': 'Jashandeep Sohi',
    'author_email': 'jashandeep.s.sohi@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/jashandeep-sohi/webcam-filters',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.9,<3.10',
}


setup(**setup_kwargs)



import gi

gi.require_version("Gst", "1.0")
gi.require_version("GstBase", "1.0")

import cv2
import numpy

from gi.repository import Gst, GstBase, GLib, GObject


GObject.threads_init()
Gst.init(None)

SRC_CAPS = Gst.Caps(Gst.Structure(
    "video/x-raw",
    format="RGB",
    width=Gst.IntRange(range(1, GLib.MAXINT)),
    height=Gst.IntRange(range(1, GLib.MAXINT)),
    framerate=Gst.FractionRange(
        Gst.Fraction(1, 1),
        Gst.Fraction(GLib.MAXINT, 1)
    )
))

SINK_CAPS = Gst.Caps(Gst.Structure(
    "video/x-raw",
    format="RGB",
    width=Gst.IntRange(range(1, GLib.MAXINT)),
    height=Gst.IntRange(range(1, GLib.MAXINT)),
    framerate=Gst.FractionRange(
        Gst.Fraction(1, 1),
        Gst.Fraction(GLib.MAXINT, 1)
    )
))

SRC_PAD_TEMPLATE = Gst.PadTemplate.new(
    "src",
    Gst.PadDirection.SRC,
    Gst.PadPresence.ALWAYS,
    SRC_CAPS
)

SINK_PAD_TEMPLATE = Gst.PadTemplate.new(
    "sink",
    Gst.PadDirection.SINK,
    Gst.PadPresence.ALWAYS,
    SINK_CAPS
)

DEFAULT_KSIZE = 10

class BoxFilter(GstBase.BaseTransform):

    __gstmetadata__ = (
        "OpenCV Boxfilter",
        "Filter",
        "Apply boxfilter",
        "Jashandeep Sohi <jashandeep.s.sohi@gmail.com>"
    )

    __gsttemplates__ = (SRC_PAD_TEMPLATE, SINK_PAD_TEMPLATE)

    __gproperties__ = {
        "ksize": (
            int,
            "ksize",
            "ksize",
            0,
            200,
            DEFAULT_KSIZE,
            GObject.ParamFlags.READWRITE
        ),
    }

    def __init__(self):
        super().__init__()
        self.ksize = DEFAULT_KSIZE
        self.set_qos_enabled(True)

    def do_get_property(self, prop: GObject.GParamSpec):
        if prop.name == "ksize":
            return self.ksize
        else:
            raise AttributeError(f"unkown property {prop.name}")

    def do_set_property(self, prop: GObject.GParamSpec, value):
        if prop.name == "ksize":
            self.ksize  = value
        else:
            raise AttributeError(f"unkown property {prop.name}")


    def do_set_caps(self, incaps, outcaps):
        s = incaps.get_structure(0)
        self.width = s.get_int("width").value
        self.height = s.get_int("height").value

        return True

    def do_transform_ip(self, inbuf):
        try:
            inbuf_info = inbuf.map(Gst.MapFlags.READ | Gst.MapFlags.WRITE)
            with inbuf_info:
                in_nd = numpy.ndarray(
                    shape=(self.height, self.width, 3),
                    dtype=numpy.uint8,
                    buffer=inbuf_info.data
                )

                in_nd[:] = cv2.boxFilter(in_nd, -1, (self.ksize, self.ksize))

                return Gst.FlowReturn.OK

        except Gst.MapError as e:
            Gst.error("mapping error %s" % e)
            return Gst.FlowReturn.ERROR
        except Exception as e:
            Gst.error("%s" % e)
            return Gst.FlowReturn.ERROR

GObject.type_register(BoxFilter)

__gstelementfactory__ = (
    "cv2_boxfilter",
    Gst.Rank.NONE,
    BoxFilter
)
